@@ -85,7 +85,7 @@ class ForkingHTTPServer(ForkingTCPServer):
             try:
                 self.server_bind()
                 self.server_activate()
-            except:
+            except BaseException:
                 self.server_close()
                 raise
 
@@ -121,9 +121,20 @@ class ForkingUnixHTTPServer(ForkingHTTPServer):
 
 
 class ForkingTLSServer(ForkingHTTPServer):
-    def __init__(self, server_address, handler_class, config, context=None):
-        ForkingHTTPServer.__init__(self, server_address, handler_class, config)
-        self._context = context if context is not None else self._mkcontext()
+    def __init__(self, server_address, handler_class, config, context=None,
+                 bind_and_activate=True):
+        ForkingHTTPServer.__init__(self, server_address, handler_class, config,
+                                   bind_and_activate=bind_and_activate)
+        if context is None:
+            try:
+                self._context = self._mkcontext()
+            except Exception as e:
+                logger.error(
+                    "Failed to create a SSLContext for TLS server: %s", e
+                )
+                raise
+        else:
+            self._context = context
 
     def _mkcontext(self):
         certfile = self.config.get('tls_certfile')
@@ -138,12 +149,19 @@ class ForkingTLSServer(ForkingHTTPServer):
         if not certfile:
             raise ValueError('tls_certfile is not set.')
 
+        logger.info(
+            "Creating SSLContext for TLS server (cafile: '%s', capath: '%s', "
+            "verify client: %s).",
+            cafile, capath, verifymode == ssl.CERT_REQUIRED
+        )
         context = ssl.create_default_context(
             ssl.Purpose.CLIENT_AUTH,
             cafile=cafile,
             capath=capath)
-        context.load_cert_chain(certfile, keyfile)
         context.verify_mode = verifymode
+        logger.info(
+            "Loading cert chain '%s' (keyfile: '%s')", certfile, keyfile)
+        context.load_cert_chain(certfile, keyfile)
         return context
 
     def get_request(self):
@@ -519,15 +537,21 @@ class HTTPServer(object):
             if not sd.is_socket_inet(fds[0], family=socket.AF_INET6,
                                      type=socket.SOCK_STREAM,
                                      listening=True, port=port):
-                raise ValueError("FD {} is not TCP IPv6 socket on port {}",
-                                 fds[0], port)
+                raise ValueError(
+                    "FD {} is not TCP IPv6 socket on port {}".format(
+                        fds[0], port
+                    )
+                )
             logger.info('Using systemd socket activation on port %i', port)
             sock = socket.fromfd(fds[0], socket.AF_INET6, socket.SOCK_STREAM)
         else:
             if not sd.is_socket_unix(fds[0], socket.SOCK_STREAM,
                                      listening=True, path=address):
-                raise ValueError("FD {} is not Unix stream socket on path {}",
-                                 fds[0], address)
+                raise ValueError(
+                    "FD {} is not Unix stream socket on path {}".format(
+                        fds[0], address
+                    )
+                )
             logger.info('Using systemd socket activation on path %s', address)
             sock = socket.fromfd(fds[0], socket.AF_UNIX, socket.SOCK_STREAM)
 
