@@ -2,6 +2,7 @@
 from __future__ import absolute_import
 
 import os
+import shlex
 import shutil
 import socket
 import subprocess
@@ -14,12 +15,17 @@ from jwcrypto import jwk
 
 import pytest
 
-from requests.exceptions import HTTPError, SSLError
+import requests.exceptions
+
+try:
+    import requests_gssapi
+except ImportError:
+    requests_gssapi = None
 
 import six
 
 from custodia.client import CustodiaKEMClient, CustodiaSimpleClient
-from custodia.compat import configparser, quote_plus
+from custodia.compat import configparser, quote_plus, unquote, urlparse
 from custodia.store.sqlite import SqliteStore
 
 
@@ -27,6 +33,13 @@ from custodia.store.sqlite import SqliteStore
 pytestmark = pytest.mark.servertest
 
 
+# requests raises any of these errors on SSL or connection error
+RequestsConnSSLErrors = (
+    requests.exceptions.SSLError,
+    requests.exceptions.ConnectionError
+)
+
+
 def find_port(host='localhost'):
     try:
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -117,6 +130,7 @@ handler = custodia.store.encgen.EncryptedOverlay
 backing_store = simple
 master_key = ${TEST_DIR}/test_mkey.conf
 master_enctype = A128CBC-HS256
+secret_protection = encrypt
 
 [authz:enc]
 handler = SimplePathAuthz
@@ -137,6 +151,8 @@ store = encgen
 
 TEST_SOCKET_URL = "http+unix://%2E%2Ftests%2Ftmp%2Ftest_socket"
 
+HERE = os.path.dirname(os.path.abspath(__file__))
+
 
 def generate_all_keys(custodia_conf):
     parser = configparser.ConfigParser(
@@ -174,10 +190,13 @@ class CustodiaTests(unittest.TestCase):
     verify_client = 'False'
     test_dir = 'tests/tmp'
 
+    maxDiff = None
+
     @classmethod
     def setUpClass(cls):
         env = os.environ.copy()
-        cls.pexec = env.get('CUSTODIAPYTHON', sys.executable)
+        # CUSTODIAPYTHON can be a list of arguments
+        cls.pexec = shlex.split(env.get('CUSTODIAPYTHON', sys.executable))
 
         if os.path.isdir(cls.test_dir):
             shutil.rmtree(cls.test_dir)
@@ -198,7 +217,7 @@ class CustodiaTests(unittest.TestCase):
         test_log_file = os.path.join(cls.test_dir, 'test_log.txt')
         with (open(test_log_file, 'a')) as logfile:
             p = subprocess.Popen(
-                [cls.pexec, '-m', 'custodia.server', custodia_conf],
+                cls.pexec + ['-m', 'custodia.server', custodia_conf],
                 env=env, stdout=logfile, stderr=logfile
             )
         time.sleep(1)
@@ -217,9 +236,7 @@ class CustodiaTests(unittest.TestCase):
         cls.kem = cls.create_client('/enc', 'kem', CustodiaKEMClient)
         cls.kem.set_server_public_keys(*srvkeys)
         cls.kem.set_client_keys(*clikeys)
-        cls.custodia_cli_args = [
-            cls.pexec,
-            '-Wignore',
+        cls.custodia_cli_args = cls.pexec + [
             '-m', 'custodia.cli',
             '--verbose',
             '--header', 'REMOTE_USER=test',
@@ -237,14 +254,47 @@ class CustodiaTests(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        cls.custodia_process.kill()
+        if cls.custodia_process.poll() is not None:
+            raise AssertionError("Custodia server crashed, see test_log.txt")
+        cls.custodia_process.terminate()
         cls.custodia_process.wait()
 
+    def check_socket(self):
+        url = urlparse(self.socket_url)
+        timeout = 0.5
+        if url.scheme == 'http+unix':
+            address = unquote(url.netloc)
+            s = socket.socket(socket.AF_UNIX)
+            s.settimeout(timeout)
+            try:
+                s.connect(address)
+            except OSError as e:
+                self.fail("Server socket unavailable: {}".format(e))
+            finally:
+                s.close()
+        else:
+            host, port = url.netloc.rsplit(":", 1)
+            try:
+                socket.create_connection(
+                    (host, int(port)), timeout=timeout
+                ).close()
+            except OSError as e:
+                self.fail("Server socket unavailable: {}".format(e))
+
+    def setUp(self):
+        if self.custodia_process.poll() is not None:
+            raise AssertionError("Custodia server crashed, see test_log.txt")
+        self.check_socket()
+
     def _custoda_cli(self, *extra_args, **kwargs):
+        env = os.environ.copy()
+        env['PYTHONWARNINGS'] = 'ignore'
         args = list(self.custodia_cli_args)
         args.extend(extra_args)
         try:
-            out = subprocess.check_output(args, stderr=subprocess.STDOUT)
+            out = subprocess.check_output(
+                args, env=env, stderr=subprocess.STDOUT,
+            )
         except subprocess.CalledProcessError as e:
             if e.returncode == 1:
                 # HTTP error, reraise
@@ -315,11 +365,11 @@ class CustodiaTests(unittest.TestCase):
         self.client.del_secret('test/http%3A%2F%2Flocalhost%3A5000')
         try:
             self.client.get_secret('test/key')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.client.last_response.status_code, 404)
         try:
             self.client.get_secret('test/http%3A%2F%2Flocalhost%3A5000')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.client.last_response.status_code, 404)
 
     def test_4_del_simple_key_cli(self):
@@ -344,13 +394,13 @@ class CustodiaTests(unittest.TestCase):
         self.fwd.delete_container('dir')
         try:
             self.admin.list_container('fwd/dir')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.admin.last_response.status_code, 404)
 
     def test_9_loop(self):
         try:
             self.loop.list_container('test')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.loop.last_response.status_code, 502)
 
     def test_A_enc_1_create_container(self):
@@ -360,7 +410,7 @@ class CustodiaTests(unittest.TestCase):
         self.enc.delete_container('container')
         try:
             self.enc.list_container('container')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.enc.last_response.status_code, 404)
 
     def test_A_enc_2_set_simple_key(self):
@@ -383,12 +433,12 @@ class CustodiaTests(unittest.TestCase):
         self.kem.del_secret('kem/key')
         try:
             self.kem.get_secret('kem/key')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.kem.last_response.status_code, 404)
         self.kem.delete_container('kem')
         try:
             self.kem.list_container('kem')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.kem.last_response.status_code, 404)
 
     def test_B_1_kem_space(self):
@@ -403,12 +453,12 @@ class CustodiaTests(unittest.TestCase):
         self.kem.del_secret('kem/key with space')
         try:
             self.kem.get_secret('kem/key with space')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.kem.last_response.status_code, 404)
         self.kem.delete_container('kem')
         try:
             self.kem.list_container('kem')
-        except HTTPError:
+        except requests.exceptions.HTTPError:
             self.assertEqual(self.kem.last_response.status_code, 404)
 
 
@@ -416,16 +466,14 @@ class CustodiaHTTPSTests(CustodiaTests):
     socket_url = 'https://localhost:{}'.format(find_port())
     verify_client = 'True'
 
-    ca_cert = 'tests/ca/custodia-ca.pem'
-    client_cert = 'tests/ca/custodia-client.pem'
-    client_key = 'tests/ca/custodia-client.key'
+    ca_cert = os.path.join(HERE, 'ca/custodia-ca.pem')
+    client_cert = os.path.join(HERE, 'ca/custodia-client.pem')
+    client_key = os.path.join(HERE, 'ca/custodia-client.key')
 
     @classmethod
     def setUpClass(cls):
         super(CustodiaHTTPSTests, cls).setUpClass()
-        cls.custodia_cli_args = [
-            cls.pexec,
-            '-Wignore',
+        cls.custodia_cli_args = cls.pexec + [
             '-m', 'custodia.cli',
             '--verbose',
             '--cafile', cls.ca_cert,
@@ -456,7 +504,8 @@ class CustodiaHTTPSTests(CustodiaTests):
     def test_client_no_ca_trust(self):
         client = CustodiaSimpleClient(self.socket_url + '/forwarder')
         client.headers['REMOTE_USER'] = 'test'
-        with self.assertRaises(SSLError) as e:
+        # XXX workaround for requests bug with urllib3 v1.22
+        with self.assertRaises(RequestsConnSSLErrors) as e:
             client.list_container('test')
         self.assert_ssl_error_msg("CERTIFICATE_VERIFY_FAILED", e.exception)
 
@@ -464,7 +513,8 @@ class CustodiaHTTPSTests(CustodiaTests):
         client = CustodiaSimpleClient(self.socket_url + '/forwarder')
         client.headers['REMOTE_USER'] = 'test'
         client.set_ca_cert(self.ca_cert)
-        with self.assertRaises(SSLError) as e:
+        # XXX workaround for requests bug with urllib3 v1.22
+        with self.assertRaises(RequestsConnSSLErrors) as e:
             client.list_container('test')
         self.assert_ssl_error_msg("SSLV3_ALERT_HANDSHAKE_FAILURE",
                                   e.exception)
@@ -480,3 +530,32 @@ class CustodiaHTTPSTests(CustodiaTests):
         self.assertEqual(key, 'VmVycnlTZWNyZXQK')
 
         self.client.del_secret('test/key')
+
+
+@pytest.mark.skipif(
+    requests_gssapi is None,
+    reason="requests_gssapi not available"
+)
+class CustodiaGSSAPITests(unittest.TestCase):
+    def test_set_gssapi_auth(self):
+        client = CustodiaSimpleClient('http://local.example')
+        self.assertEqual(client.session.auth, None)
+
+        client.set_gssapi_auth()
+        self.assertIsInstance(
+            client.session.auth,
+            requests_gssapi.HTTPSPNEGOAuth
+        )
+        self.assertEqual(
+            client.session.auth.opportunistic_auth,
+            False
+        )
+        client.set_gssapi_auth(opportunistic_auth=True)
+        self.assertIsInstance(
+            client.session.auth,
+            requests_gssapi.HTTPSPNEGOAuth
+        )
+        self.assertEqual(
+            client.session.auth.opportunistic_auth,
+            True
+        )
